package com.cal.sca.service.command.commandImpl;

import com.cal.sca.entity.Cart;
import com.cal.sca.entity.CartItem;
import com.cal.sca.entity.Product;
import com.cal.sca.model.service.AddProductToCartRequest;
import com.cal.sca.repository.CartRepository;
import com.cal.sca.repository.ProductRepository;
import com.cal.sca.service.AbstractCommand;
import com.cal.sca.service.command.AddProductToCartCommand;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

import java.util.List;

@Component
public class AddProductToCartCommandImpl extends AbstractCommand<Cart, AddProductToCartRequest> implements AddProductToCartCommand {

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private CartRepository cartRepository;

    @Override
    public Mono<Cart> doExecute(AddProductToCartRequest request) {
        return Mono.zip(
                objects -> addOrUpdateProductInCart((Cart) objects[0], (Product) objects[1], request.getQuantity()),
                cartRepository.findById(request.getCartId()),
                productRepository.findById(request.getProductId())
        ).flatMap(cart -> cartRepository.save(cart));
    }

    @Override
    public Mono<List<Cart>> doExecuteList(AddProductToCartRequest addProductToCartRequest) {
        return null;
    }

    private Cart addOrUpdateProductInCart(Cart cart, Product product, Integer quantity) {

        if(isCartContainProduct(cart, product)) {
            incrementProductQuantity(cart, product, quantity);
        } else {
            addNewProductToCart(cart, product, quantity);
        }

        return cart;
    }

    private boolean isCartContainProduct(Cart cart, Product product) {
        return cart.getItems().stream()
                .anyMatch(cartItem -> cartItem.getId().equals(product.getId()));
    }

    private void incrementProductQuantity(Cart cart, Product product, Integer quantity) {
        cart.getItems().stream()
                .filter(cartItem -> cartItem.getId().equals(product.getId()))
                .forEach(cartItem -> cartItem.setQuantity(cartItem.getQuantity() + quantity));
    }

    private void addNewProductToCart(Cart cart, Product product, Integer quantity) {
        CartItem item = CartItem.builder()
                .id(product.getId())
                .name(product.getName())
                .price(product.getPrice())
                .quantity(quantity)
                .build();

        cart.getItems().add(item);
    }
}
